<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'trustbeautyreviews/vendor/autoload.php';

class AdminTrustbeautyreviewsSettingsController extends ModuleAdminController
{
    /**
     * @var bool
     */
    public $bootstrap = true;

    /**
     * @var Module
     */
    public $module;

    /**
     * @var string
     */
    public $className = Configuration::class;

    /**
     * @var string
     */
    public $table = 'configuration';

    /**
     * @var TrustbeautyreviewsAddon\Service\AuthService
     */
    protected $authService;

    public function __construct()
    {
        parent::__construct();
        $this->authService = new TrustbeautyreviewsAddon\Service\AuthService();
    }

    protected function renderAuthForm()
    {
        $this->fields_options = [
            'auth' => [
                'image' => '../img/admin/cog.gif',
                'title' => $this->module->l('Module activation', 'AdminTrustbeautyreviewsSettings'),
                'fields' => [
                    TrustbeautyReviews::APIKEY => [
                        'type' => 'text',
                        'title' => $this->module->l('API key: ', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l(
                            'API key to access Trustbeauty ressources.',
                            'AdminTrustbeautyreviewsSettings'
                        ),
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->module->l('Save', 'Admin.Global'),
                    'name' => 'submitAuthForm',
                ],
            ],
        ];
    }

    protected function renderSettingsForm()
    {
        $this->fields_options = [
            'products_overview' => [
                'image' => '../img/admin/cog.gif',
                'title' => $this->module->l('Product overviews', 'AdminTrustbeautyreviewsSettings'),
                'fields' => [
                    TrustbeautyReviews::PRODUCTS_AVG_RATING => [
                        'type' => 'bool',
                        'title' => $this->module->l('Average grade :', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l(
                            'Show average rating in product overviews.',
                            'AdminTrustbeautyreviewsSettings'
                        ),
                    ],
                    TrustbeautyReviews::PRODUCTS_NB_REVIEWS => [
                        'type' => 'bool',
                        'title' => $this->module->l('Number of reviews :', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l(
                            'Show number of reviews in product previews',
                            'AdminTrustbeautyreviewsSettings'
                        ),
                    ],
                ],
                'submit' => [
                    'title' => $this->module->l('Save', 'Admin.Global'),
                    'name' => 'submitProductsOverviewForm',
                ],
            ],
            'product_overview' => [
                'image' => '../img/admin/cog.gif',
                'title' => $this->module->l('Product sheet - Summary of reviews', 'AdminTrustbeautyreviewsSettings'),
                'fields' => [
                    TrustbeautyReviews::PRODUCT_AVG_RATING => [
                        'type' => 'bool',
                        'title' => $this->module->l('Average grade :', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l(
                            'Display the average review score under the product name',
                            'AdminTrustbeautyreviewsSettings'
                        ),
                    ],
                    TrustbeautyReviews::PRODUCT_NB_REVIEWS => [
                        'type' => 'bool',
                        'title' => $this->module->l('Number of reviews :', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l(
                            'Display the number of reviews under the product name',
                            'AdminTrustbeautyreviewsSettings'
                        ),
                    ],
                ],
                'submit' => [
                    'title' => $this->module->l('Save', 'Admin.Global'),
                    'name' => 'submitProductOverviewForm',
                ],
            ],
            'list_overviews' => [
                'image' => '../img/admin/cog.gif',
                'title' => $this->module->l('Product sheet - List of reviews', 'AdminTrustbeautyreviewsSettings'),
                'fields' => [
                    TrustbeautyReviews::WIDGET_NB_COLUMNS => [
                        'type' => 'select',
                        'title' => $this->module->l('Number of reviews per line :', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l('Number of reviews per line', 'AdminTrustbeautyreviewsSettings'),
                        'list' => [
                            ['id' => 1, 'name' => 1],
                            ['id' => 2, 'name' => 2],
                        ],
                        'identifier' => 'id',
                    ],
                    TrustbeautyReviews::WIDGET_NB_REVIEWS => [
                        'type' => 'select',
                        'title' => $this->module->l('Number of reviews :', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l('Number of reviews per page', 'AdminTrustbeautyreviewsSettings'),
                        'list' => [
                            ['id' => 4, 'name' => 4],
                            ['id' => 6, 'name' => 6],
                            ['id' => 8, 'name' => 8],
                            ['id' => 10, 'name' => 10],
                            ['id' => 12, 'name' => 12],
                            ['id' => 24, 'name' => 24],
                        ],
                        'identifier' => 'id',
                    ],
                    TrustbeautyReviews::WIDGET_ORDER => [
                        'type' => 'select',
                        'title' => $this->module->l('Default order :', 'AdminTrustbeautyreviewsSettings'),
                        'desc' => $this->module->l('Default order of reviews', 'AdminTrustbeautyreviewsSettings'),
                        'list' => [
                            ['id' => 'date_desc', 'name' => $this->module->l(
                                'The most recent',
                                'AdminTrustbeautyreviewsSettings'
                            )],
                            ['id' => 'influencer_desc', 'name' => $this->module->l(
                                'Beauty expert',
                                'AdminTrustbeautyreviewsSettings'
                            )],
                            ['id' => 'useful_desc', 'name' => $this->module->l(
                                'Most useful',
                                'AdminTrustbeautyreviewsSettings'
                            )],
                            ['id' => 'rating_desc', 'name' => $this->module->l(
                                'Highest score',
                                'AdminTrustbeautyreviewsSettings'
                            )],
                        ],
                        'identifier' => 'id',
                    ],
                ],
                'submit' => [
                    'title' => $this->module->l('Save', 'Admin.Global'),
                    'name' => 'submitListOverviewsForm',
                ],
            ],
        ];
    }

    private function getStatutForm()
    {
        $orderStates = $this->listOrderStates();
        $fields_form = [
            'form' => [
                'form' => [
                    'id_form' => 'order-statut-form',
                    'legend' => [
                        'title' => $this->module->l('Reviews after purchase', 'AdminTrustbeautyreviewsSettings'),
                    ],
                    'description' => $this->module->l(
                        'An email will be sent by Trustt to your customers to write a review on the purchased products.
                        Please select below the order states that will trigger this email.',
                        'AdminTrustbeautyreviewsSettings'
                    ),
                    'input' => [
                        [
                            'type' => 'double-list',
                            'name' => 'order-statut-values',
                            'title' => $this->module->l('Order states', 'AdminTrustbeautyreviewsSettings'),
                            'desc' => $this->module->l('Order states', 'AdminTrustbeautyreviewsSettings'),
                            'unselected' => [
                                'id' => 'status_order_add',
                                'label' => $this->module->l(
                                    'Unselected order states',
                                    'AdminTrustbeautyreviewsSettings'
                                ),
                                'options' => $orderStates['unselected'],
                                'btn' => [
                                    'id' => 'status_order_add_btn',
                                    'label' => $this->module->l('Add', 'AdminTrustbeautyreviewsSettings'),
                                ],
                            ],
                            'selected' => [
                                'id' => 'status_order_remove',
                                'label' => $this->module->l('Selected order states', 'AdminTrustbeautyreviewsSettings'),
                                'options' => $orderStates['selected'],
                                'btn' => [
                                    'id' => 'status_order_remove_btn',
                                    'label' => $this->module->l('Remove', 'AdminTrustbeautyreviewsSettings'),
                                ],
                            ],
                        ],
                    ],
                    'submit' => [
                        'title' => $this->module->l('Save', 'Admin.Global'),
                        'name' => 'submitListStatusForm',
                    ],
                ],
            ],
        ];

        $helper = new HelperForm($this);
        $helper->base_folder = $this->getTemplatePath() . $this->override_folder;
        $helper->base_tpl = 'order-statut-form.tpl';

        return $helper->generateForm($fields_form);
    }

    protected function listOrderStates()
    {
        $orderStates = ['selected' => [], 'unselected' => []];
        $idLanguage = Context::getContext()->language->id;
        $statesStatus = OrderState::getOrderStates($idLanguage);
        $selectedOrders = Configuration::get(TrustbeautyReviews::ORDERS_STATUS);
        $aOrdersSel = json_decode($selectedOrders, true);
        if ($aOrdersSel == null) {
            $aOrdersSel = [];
        }
        foreach ($statesStatus as $aState) {
            if (!$aState['hidden'] && !$aState['deleted']) {
                $sSelected = in_array($aState['id_order_state'], $aOrdersSel) !== false ? 'selected' : 'unselected';
                $orderStates[$sSelected][] = [
                    'value' => $aState['id_order_state'],
                    'label' => $aState['name'],
                ];
            }
        }

        return $orderStates;
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitProductsOverviewForm')) {
            $this->submitProductsOverviewForm();
        } elseif (Tools::isSubmit('submitProductOverviewForm')) {
            $this->submitProductOverviewForm();
        } elseif (Tools::isSubmit('submitListOverviewsForm')) {
            $this->submitListOverviewsForm();
        } elseif (Tools::isSubmit('submitListStatusForm')) {
            $this->submitListStatusForm();

            return true;
        } elseif (Tools::isSubmit('submitAuthForm')) {
            $this->submitAuthForm();
        }

        return parent::postProcess();
    }

    public function renderOptions()
    {
        $apiKey = Configuration::get(TrustbeautyReviews::APIKEY);
        $isApiChecked = Configuration::get(TrustbeautyReviews::API_CHECKED) == 'true';

        if ($isApiChecked === true) {
            $this->addJS($this->module->getPathUri() . 'views/js/trustbeautyreviews-backoffice.js');
            $this->renderSettingsForm();
        } else {
            $this->renderAuthForm();
        }

        $options = parent::renderOptions();

        if (!$isApiChecked) {
            $tpl = Context::getContext()->smarty->createTemplate(
                _PS_MODULE_DIR_ . 'trustbeautyreviews/views/templates/admin/auth.tpl'
            );
            $tpl->assign([
                'apiKeyIsValid' => empty($apiKey),
            ]);

            return $tpl->fetch() . $options;
        } else {
            $options .= $this->getStatutForm();
        }

        return $options;
    }

    //region Save Fields

    protected function submitProductsOverviewForm()
    {
        $showAvgRating = Tools::getValue(TrustbeautyReviews::PRODUCTS_AVG_RATING);
        $showNbReviews = Tools::getValue(TrustbeautyReviews::PRODUCTS_NB_REVIEWS);

        Configuration::updateValue(TrustbeautyReviews::PRODUCTS_AVG_RATING, $showAvgRating);
        Configuration::updateValue(TrustbeautyReviews::PRODUCTS_NB_REVIEWS, $showNbReviews);

        $this->confirmations = $this->_conf[4];
    }

    protected function submitProductOverviewForm()
    {
        $showAvgRating = Tools::getValue(TrustbeautyReviews::PRODUCT_AVG_RATING);
        $showNbReviews = Tools::getValue(TrustbeautyReviews::PRODUCT_NB_REVIEWS);

        Configuration::updateValue(TrustbeautyReviews::PRODUCT_AVG_RATING, $showAvgRating);
        Configuration::updateValue(TrustbeautyReviews::PRODUCT_NB_REVIEWS, $showNbReviews);

        $this->confirmations = $this->_conf[4];
    }

    protected function submitListOverviewsForm()
    {
        $widgetNbColumns = Tools::getValue(TrustbeautyReviews::WIDGET_NB_COLUMNS);
        $widgetNbReviews = Tools::getValue(TrustbeautyReviews::WIDGET_NB_REVIEWS);
        $widgetOrder = Tools::getValue(TrustbeautyReviews::WIDGET_ORDER);

        Configuration::updateValue(TrustbeautyReviews::WIDGET_NB_COLUMNS, $widgetNbColumns);
        Configuration::updateValue(TrustbeautyReviews::WIDGET_NB_REVIEWS, $widgetNbReviews);
        Configuration::updateValue(TrustbeautyReviews::WIDGET_ORDER, $widgetOrder);

        $this->confirmations = $this->_conf[4];
    }

    protected function submitListStatusForm()
    {
        $listStates = Tools::getValue('order-statut-values');
        if (!$listStates) {
            $listStates = [];
        }

        Configuration::updateValue(TrustbeautyReviews::ORDERS_STATUS, json_encode($listStates));

        $this->confirmations = $this->_conf[4];
    }

    protected function submitAuthForm()
    {
        $this->authService->processAuth(Tools::getValue(Trustbeautyreviews::APIKEY));
        $this->confirmations = $this->_conf[4];
    }

    //endregion
}
