<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

namespace TrustbeautyreviewsAddon\API;

use TrustbeautyreviewsAddon\API\Request\AbstractRequest;
use TrustbeautyreviewsAddon\API\Response\ClientResponse;

class Client
{
    /**
     * @param $request AbstractRequest
     *
     * @return array|false
     */
    public function call($request)
    {
        if (function_exists('curl_init') == false) {
            return false;
        }

        $url = $request->getUrl();

        if (isset($_SERVER['HTTP_REFERER']) && $_SERVER['HTTP_REFERER'] != '') {
            $referer = $_SERVER['HTTP_REFERER'];
        } else {
            $referer = $request->getParams()['host'];
        }

        $curl = curl_init();
        curl_setopt_array(
            $curl,
            [
                CURLOPT_URL => $url,
                CURLOPT_SSL_VERIFYPEER => 0,
                CURLOPT_REFERER => $referer,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => http_build_query($request->getParams()),
                CURLOPT_HTTPHEADER => [
                    'cache-control: no-cache',
                ],
            ]
        );

        $response = curl_exec($curl);
        $err = curl_error($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        $response = new ClientResponse([
            'httpCode' => $httpCode,
            'content' => $response,
            'error' => $err,
        ]);

        return $response->getResponse();
    }
}
