<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

namespace TrustbeautyreviewsAddon\API\Request;

use Configuration;
use TrustbeautyReviews;

abstract class AbstractRequest
{
    const URL_API = 'https://app.trustt.io/api/';

    protected $params = [];

    abstract public function getMethod();

    public function getParams()
    {
        return array_merge($this->params, $this->getDefaultParams());
    }

    protected function getDefaultParams()
    {
        return [
            'host' => $this->getHttpHost(),
            'api_key' => Configuration::get(TrustbeautyReviews::APIKEY),
        ];
    }

    public function addParam($key, $value)
    {
        $this->params[$key] = $value;

        return $this;
    }

    /**
     * @TODO Optimize it
     *
     * @return string
     */
    protected function getHttpHost()
    {
        $httpHost = '';
        if (isset($_SERVER['HTTP_HOST']) && $_SERVER['HTTP_HOST'] != '') {
            if (isset($_SERVER['HTTPS'])) {
                $protocol = ($_SERVER['HTTPS'] && $_SERVER['HTTPS'] != 'off') ? 'https' : 'http';
            } else {
                $protocol = 'http';
            }
            $httpHost = $protocol . '://' . $_SERVER['HTTP_HOST'];
        }

        return $httpHost;
    }

    public function getUrl()
    {
        return self::URL_API . $this->getMethod();
    }
}
