<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

namespace TrustbeautyreviewsAddon\Action;

use Context;
use Db;
use DbQuery;
use Image;
use Product;

class GetProductsAction implements ActionInterface
{
    public function handle($params = [])
    {
        $productIds = '';
        $limit = 0;
        $idLang = Context::getContext()->language->id;

        if (!empty($params['ids'])) {
            $productIds = $params['ids'];
        }

        return $this->getProductsInfo($idLang, $productIds, $limit);
    }

    private function filterIds($ids)
    {
        return array_filter($ids, function ($id) {
            return is_numeric($id);
        });
    }

    protected function getProductsInfo($idLang, $productIds = '', $limit = 0)
    {
        if ($productIds != '') {
            $ids = explode(',', $productIds);
            $validProductIds = array_map(function ($id) {
                return (int) $id;
            }, $this->filterIds($ids));
            $productIds = implode(',', $validProductIds);
        }

        $query = new DbQuery();
        $query->select('id_product')
            ->from(Product::$definition['table']);

        if (!empty($productIds)) {
            $query->where('id_product IN (' . $productIds . ')');
        }

        $query->orderBy('`date_add` DESC');

        if ($limit > 0) {
            $query->limit((int) $limit);
        }

        $productIds = Db::getInstance()->executeS($query);

        // get products infos
        $products = [];
        foreach ($productIds as $product) {
            $productId = $product['id_product'];
            $product = new Product($productId, false, $idLang);

            $pricePublic = $product->getPriceWithoutReduct();

            $coverImage = Image::getCover($productId);
            if (version_compare(_PS_VERSION_, '1.7.7.0', '>=')) {
                $homeImg = \ImageType::getFormattedName('home'); // Exception in 1.6
            } else {
                $homeImg = \ImageType::getFormatedName('home');
            }
            $coverImagePath = Context::getContext()->link->getImageLink(
                $product->link_rewrite,
                $coverImage['id_image'],
                $homeImg
            );
            $productURL = Context::getContext()->link->getProductLink($product);

            $images = $product->getImages($idLang);
            $productImages = [];
            foreach ($images as $image) {
                $imagePath = Context::getContext()->link->getImageLink(
                    $product->link_rewrite,
                    $image['id_image'],
                    $homeImg
                );

                $productImages[] = [
                    'id_image' => $image['id_image'],
                    'cover' => $image['cover'],
                    'legend' => $image['legend'],
                    'position' => $image['position'],
                    'url' => $imagePath,
                ];
            }

            $infos = [
                'product' => $product,
                'pricePublic' => \Tools::ps_round($pricePublic, 2),
                'url' => $productURL,
                'cover' => $coverImagePath,
                'images' => $productImages,
            ];

            $products[] = $infos;
        }

        return $products;
    }
}
