<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to a commercial license from SARL 202 ecommence
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the SARL 202 ecommence is strictly forbidden.
 * In order to obtain a license, please contact us: tech@202-ecommerce.com
 * ...........................................................................
 * INFORMATION SUR LA LICENCE D'UTILISATION
 *
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe 202 ecommence
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence ecrit de la part de la SARL 202 ecommence est
 * expressement interdite.
 * Pour obtenir une licence, veuillez contacter 202-ecommerce <tech@202-ecommerce.com>
 * ...........................................................................
 *
 * @author    202-ecommerce <tech@202-ecommerce.com>
 * @copyright Copyright (c) 202-ecommerce
 * @license   Commercial license
 */

namespace TrustbeautyreviewsAddon\Hook;

use Configuration;
use Context;
use Product;
use Tools;
use TrustbeautyReviews;
use TrustbeautyreviewsAddon\Service\ProductService;
use TrustbeautyreviewsClasslib\Hook\AbstractHook;

class HookProduct extends AbstractHook
{
    const AVAILABLE_HOOKS = [
        'displayProductTabContent',
        'displayProductListReviews',
        'displayProductExtraBlock',
    ];

    /** @var string */
    protected $file = _PS_MODULE_DIR_ . 'trustbeautyreviews';

    /** @var string */
    protected $path = __PS_BASE_URI__ . '/modules/trustbeautyreview/';

    /** @var int */
    protected $langIsoCode;

    /** @var int */
    protected $productId;

    /** @var Product */
    protected $product;

    /** @var int */
    protected $shopId;

    /** @var string */
    protected $cache_id;

    /** @var bool */
    protected $apiKeyChecked;

    /**
     * @param int $productId
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    protected function initVariables($productId)
    {
        $this->productId = $productId > 0 ? (int) $productId : (int) Tools::getValue('id_product');
        $this->langIsoCode = (int) Context::getContext()->language->iso_code;
        $this->shopId = (int) Context::getContext()->shop->id;
        $this->apiKeyChecked = Configuration::get(TrustbeautyReviews::API_CHECKED) == 'true';
    }

    private function assignProduct()
    {
        $this->product = new Product($this->productId, false, Context::getContext()->cookie->id_lang);
        if (empty($this->product->getAccessories(Context::getContext()->cookie->id_langd))) {
            $this->product->accessories = [];
        }
    }

    public function displayProductListReviews($params)
    {
        $productId = (int) $params['product']['id_product'];

        $this->initVariables((int) $productId);

        if (!$this->apiKeyChecked) {
            return;
        }

        $showAvgRating = Configuration::get(TrustbeautyReviews::PRODUCTS_AVG_RATING);
        $showNbReviews = Configuration::get(TrustbeautyReviews::PRODUCTS_NB_REVIEWS);

        $this->getCacheId('-product-listreviews' . $showAvgRating . $showNbReviews);

        $this->processProductListReviews();
        $this->assignProductListReviews($showAvgRating, $showNbReviews);

        return $this->module->display($this->file, 'displayProductListReviews.tpl', $this->cache_id);
    }

    protected function processProductListReviews()
    {
    }

    protected function assignProductListReviews($showAvgRating, $showNbReviews)
    {
        if ($this->module->isCached('displayProductListReviews.tpl', $this->cache_id)) {
            return;
        }

        $this->assignProduct();

        /** @var ProductService $productService */
        $productService = new ProductService();

        list($nbReviews, $avgRating) = $productService->getProductStats(
            $this->productId,
            $this->shopId,
            $this->langIsoCode
        );

        Context::getContext()->smarty->assign([
            'showAvgRating' => $showAvgRating,
            'showNbReviews' => $showNbReviews,
            'nbComments' => $nbReviews,
            'avgRating' => round($avgRating, 1),
            'avgRating100' => $productService->normalizeRating100($avgRating),
            'product' => $this->product,
        ]);
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            Context::getContext()->smarty->assign('base_dir', _PS_BASE_URL_);
        }

        $productURL = Context::getContext()->link->getProductLink($this->product);
        Context::getContext()->smarty->assign('productURL', $productURL);
    }

    public function displayProductExtraBlock()
    {
        $this->initVariables(-1); // -1 : Init by Parameter Tools::getValue('id_product')

        if (!$this->apiKeyChecked) {
            return '';
        }

        $this->processProductExtraBlock();
        $this->assignProductExtraBlock();

        return $this->module->display(
            $this->file,
            'displayProductExtraBlock.tpl',
            $this->cache_id
        );
    }

    protected function processProductExtraBlock()
    {
    }

    protected function assignProductExtraBlock()
    {
        $showAvgRating = Configuration::get(TrustbeautyReviews::PRODUCT_AVG_RATING);
        $showNbReviews = Configuration::get(TrustbeautyReviews::PRODUCT_NB_REVIEWS);

        $this->getCacheId('-product-extrablock' . $showAvgRating . $showNbReviews);

        if ($this->module->isCached('displayProductExtraBlock.tpl', $this->cache_id)) {
            return;
        }

        $this->assignProduct();

        /** @var ProductService $prodService */
        $productService = new ProductService();

        list($nbReviews, $avgRating) = $productService->getProductStats(
            $this->productId,
            $this->shopId,
            $this->langIsoCode
        );

        Context::getContext()->smarty->assign([
            'showAvgRating' => $showAvgRating,
            'showNbReviews' => $showNbReviews,
            'nbComments' => $nbReviews,
            'avgRating' => round($avgRating, 1),
            'avgRating100' => $productService->normalizeRating100($avgRating),
            'product' => $this->product,
        ]);
    }

    public function displayProductTabContent()
    {
        $this->initVariables(-1); // -1 : Init by Parameter Tools::getValue('id_product')

        if (!$this->apiKeyChecked) {
            return '';
        }

        $this->processProductTabContent();
        $this->assignProductTabContent();

        return $this->module->display($this->file, 'displayProductTabContent.tpl', $this->cache_id);
    }

    protected function processProductTabContent()
    {
    }

    protected function assignProductTabContent()
    {
        $nbColumns = Configuration::get(TrustbeautyReviews::WIDGET_NB_COLUMNS);
        $nbReviews = Configuration::get(TrustbeautyReviews::WIDGET_NB_REVIEWS);
        $order = Configuration::get(TrustbeautyReviews::WIDGET_ORDER);

        $this->getCacheId('-product_tabcontent' . $nbColumns . $nbReviews . $order);

        Context::getContext()->controller->addCSS($this->path . 'views/css/trustbeautyreviews.css', 'all');
        Context::getContext()->controller->addJS($this->path . 'views/js/trustbeautyreviews.js');

        if ($this->module->isCached('displayProductTabContent.tpl', $this->cache_id)) {
            return;
        }

        $this->assignProduct();

        $showWidget = $this->productId > 0;

        Context::getContext()->smarty->assign('showWidget', $showWidget);

        if (!$showWidget) {
            return;
        }

        $apiKey = Configuration::get(TrustbeautyReviews::APIKEY);

        Context::getContext()->smarty->assign([
            'apiKey' => $apiKey,
            'product' => $this->product,
            'shopId' => $this->shopId,
            'langIsoCode' => $this->langIsoCode,
            'widgetNbColumns' => $nbColumns,
            'widgetNbReviews' => $nbReviews,
            'widgetOrder' => $order,
        ]);
    }

    /**
     * @param string $block
     */
    protected function getCacheId($block)
    {
        $cache_id = $this->module->smartyGetCacheId($this->module->name)
            . $this->langIsoCode . $block . '_' . $this->productId;

        $this->cache_id = $cache_id;
    }
}
