<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

namespace TrustbeautyreviewsAddon\Service;

use Exception;
use PrestaShopLogger;
use ProductStat;
use TrustbeautyreviewsAddon\API\Client;
use TrustbeautyreviewsAddon\API\Request\ProductRequest;
use TrustbeautyreviewsAddon\Repository\ProductRepository;

class ProductService
{
    const API_STATS_LIFETIME = 2; // in minutes

    /** @var ProductRepository */
    private $productRepository;

    public function __construct()
    {
        $this->productRepository = new ProductRepository();
    }

    public function getProductStats($idProduct, $idShop, $isoCode = null)
    {
        $idProduct = (int) $idProduct;
        $idShop = (int) $idShop;

        if (is_null($isoCode)) {
            $isoCode = '0';
        }

        $nbReviews = 0;
        $avgRating = 0.0;
        $results = $this->productRepository->getStats($idProduct, $isoCode, $idShop, self::API_STATS_LIFETIME);

        if ($results) {
            $nbReviews = $results['nb_reviews'];
            $avgRating = $results['avg_rating'];
        }

        if (!$results) {
            $client = new Client();
            $request = new ProductRequest();
            $request->addParam('id_product', $idProduct)
                ->addParam('id_shop', $idShop)
                ->addParam('iso_code', $isoCode);

            $response = $client->call($request);
            if ($response['code'] == 200) {
                $json = json_decode($response['response'], true);

                if (isset($json['nbReviews']) && isset($json['avgRating'])) {
                    $nbReviews = $json['nbReviews'];
                    $avgRating = $json['avgRating'];
                }
            }
            // Case New Product Stat
            $productStat = new ProductStat();

            // Test if already exist (isolang + idproduct + idshop)
            $productStatIn = $this->productRepository->getStats($idProduct, $isoCode, $idShop, 0);
            if ($productStatIn && isset($productStatIn['id_trustbeautyreviews_product_stat'])) {
                // We will keep old stats if product found but no response
                $productStat = new ProductStat($productStatIn['id_trustbeautyreviews_product_stat']);
                if ($nbReviews == 0) {
                    $nbReviews = $productStatIn['nb_reviews'];
                    $avgRating = $productStatIn['avg_rating'];
                }
            }
            // In all cases save data
            $productStat->id_product = $idProduct;
            $productStat->iso_lang = $isoCode;
            $productStat->id_shop = $idShop;
            $productStat->nb_reviews = $nbReviews;
            $productStat->avg_rating = $avgRating;
            try {
                $productStat->save();
            } catch (Exception $ex) {
                PrestaShopLogger::addLog('Error Database Add : ' . $ex->getMessage());
            }
        }

        return [$nbReviews, $avgRating];
    }

    public function normalizeRating100($rating)
    {
        return 100 * (round($rating * 4) / 4);
    }
}
