<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'trustbeautyreviews/vendor/autoload.php';

/**
 * To let module compatible with Prestashop 1.6 please keep this following line commented in PrestaShop 1.6:
 * // use TrustbeautyreviewsClasslib\Install\ModuleInstaller;
 * // use TrustbeautyreviewsClasslib\Actions\ActionsHandler;
 * // use TrustbeautyreviewsClasslib\Module;
 * // use TrustbeautyreviewsAddon\Hook\HookDispatcher;
 *
 * Developers use declarative method to define objects, parameters, controllers... needed in this module
 */

class TrustbeautyReviews extends TrustbeautyreviewsClasslib\Module
{
    /** @var string name */
    public $name = 'trustbeautyreviews';

    /** @var string tab */
    public $tab = 'advertising_marketing';

    /** @var string autor */
    public $autor = 'Trustbeauty';

    /** @var string version */
    public $version = '1.0.0';

    /** @var int need_instance */
    public $need_instance = 0;

    /** @var array filled with known compliant PS versions */
    public $ps_versions_compliancy = [
        'min' => '1.6',
        'max' => '1.7.9.99',
    ];

    /** Extensions
     *
     * @var array
     */
    public $extensions = [];

    /** @var string This module requires at least PHP version */
    public $php_version_required = '5.6';

    public $controllers = [
        'api',
    ];

    public $moduleAdminControllers = [
        [
            'name' => [
                'en' => 'Configuration',
                'fr' => 'Configuration',
            ],
            'class_name' => 'AdminTrustbeautyreviewsSettings',
            'parent_class_name' => 'trustbeautyreviews',
            'visible' => false,
        ],
    ];

    public $objectModels = [
        ProductStat::class,
    ];

    /**
     * List of hooks used in this Module
     *
     * @var array
     */
    public $hooks = [];

    /**
     * List of cron tasks indexed by controller name
     * Title value must be an array indexed by iso language (en is required)
     * Frequency value can be hourly, daily, weekly, monthly
     *
     * @var array
     */
    public $cronTasks = [];

    //region Constants

    const PRODUCTS_AVG_RATING = 'TRUSTBEAUTYREVIEWS_PRODUCTS_AVG_RATING';

    const PRODUCTS_NB_REVIEWS = 'TRUSTBEAUTYREVIEWS_PRODUCTS_NB_REVIEWS';

    const PRODUCT_AVG_RATING = 'TRUSTBEAUTYREVIEWS_PRODUCT_AVG_RATING';

    const PRODUCT_NB_REVIEWS = 'TRUSTBEAUTYREVIEWS_PRODUCT_NB_REVIEWS';

    const WIDGET_NB_COLUMNS = 'TRUSTBEAUTYREVIEWS_WIDGET_NB_COLUMNS';

    const WIDGET_NB_REVIEWS = 'TRUSTBEAUTYREVIEWS_WIDGET_NB_REVIEWS';

    const WIDGET_ORDER = 'TRUSTBEAUTYREVIEWS_WIDGET_ORDER';

    const APIKEY = 'TRUSTBEAUTYREVIEWS_APIKEY';

    const API_CHECKED = 'TRUSTBEAUTYREVIEWS_API_CHECKED';

    const ORDERS_STATUS = 'TRUSTBEAUTYREVIEWS_ORDERS_STATUS';

    const VERSION = '1.0.0';

    //endregion

    public function __construct()
    {
        $this->name = 'trustbeautyreviews';
        $this->tab = 'advertising_marketing';
        $this->author = 'Trustt';
        $this->version = '1.0.0';
        parent::__construct();
        $this->bootstrap = true;
        $this->displayName = $this->l('Trustt Reviews');
        $this->description = $this->l('Products reviews may increase your sells. 
        Collect them with the Trustt services');
        $this->hookDispatcher = new TrustbeautyreviewsAddon\Hook\HookDispatcher($this);
        $this->hooks = array_merge($this->hooks, $this->hookDispatcher->getAvailableHooks());
    }

    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        Configuration::updateGlobalValue(self::PRODUCTS_AVG_RATING, '1');
        Configuration::updateGlobalValue(self::PRODUCTS_NB_REVIEWS, '1');

        Configuration::updateGlobalValue(self::PRODUCT_AVG_RATING, '1');
        Configuration::updateGlobalValue(self::PRODUCT_NB_REVIEWS, '1');

        Configuration::updateGlobalValue(self::WIDGET_NB_COLUMNS, '2');
        Configuration::updateGlobalValue(self::WIDGET_NB_REVIEWS, '6');
        Configuration::updateGlobalValue(self::WIDGET_ORDER, 'date_desc');

        Configuration::updateGlobalValue(self::APIKEY, '');
        Configuration::updateGlobalValue(self::API_CHECKED, '');
        Configuration::updateGlobalValue(
            self::ORDERS_STATUS,
            json_encode([Configuration::get('PS_OS_DELIVERED')]) // Status (id) livré
        );

        $this->addSQLUniqueKey();

        return true;
    }

    private function addSQLUniqueKey()
    {
        $newTable = _DB_PREFIX_ . ProductStat::$definition['table'];
        $sqlCopy = "ALTER TABLE `{$newTable}` ADD UNIQUE KEY `ProductShopLang` (`id_product`,`id_shop`,`iso_lang`);";
        Db::getInstance()->execute($sqlCopy);
    }

    public function onClickOption($type)
    {
        $confirm_reset = $this->l('Reseting this module will delete all comments from your database
        , are you sure you want to reset it ?');
        $reset_callback = "return trustbeautyreviews_reset('" . addslashes($confirm_reset) . "');";

        $matchType = [
            'reset' => $reset_callback,
            'delete' => "return confirm('Confirm delete?')",
        ];

        if (isset($matchType[$type])) {
            return $matchType[$type];
        }

        return '';
    }

    public function getContent()
    {
        Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminTrustbeautyreviewsSettings'));
    }

    public function smartyGetCacheId($name = null)
    {
        return $this->getCacheId($name);
    }

    public function smartyClearCache($template, $cache_id = null, $compile_id = null)
    {
        return $this->_clearCache($template, $cache_id, $compile_id);
    }

    /**
     * Handle Hooks loaded on extension
     *
     * @param string $name Hook name
     * @param array $arguments Hook arguments
     *
     * @return mixed
     */
    public function __call($name, $arguments)
    {
        if ($result = $this->handleExtensionsHook(
            $name,
            !empty($arguments[0]) ? $arguments[0] : []
        )) {
            if (!is_null($result)) {
                return $result;
            }
        }
    }
}
