<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

use TrustbeautyreviewsClasslib\Install\ModuleInstaller;

/**
 * @param TrustbeautyReviews $module
 *
 * @return bool
 */
function upgrade_module_1_0_0($module)
{
    $moduleInstaller = new ModuleInstaller($module);
    $moduleInstaller->uninstallObjectModels();

    $moduleInstaller->installObjectModels();

    copyDataOldTable();

    dropOldTables();

    updateConfigurationName('TBR_PRODUCTS_AVGRATING', TrustbeautyReviews::PRODUCTS_AVG_RATING);
    updateConfigurationName('TBR_PRODUCTS_NBREVIEWS', TrustbeautyReviews::PRODUCTS_NB_REVIEWS);
    updateConfigurationName('TBR_PRODUCT_AVGRATING', TrustbeautyReviews::PRODUCT_AVG_RATING);
    updateConfigurationName('TBR_PRODUCT_NBREVIEWS', TrustbeautyReviews::PRODUCT_NB_REVIEWS);
    updateConfigurationName('TBR_WIDGET_NBCOLUMNS', TrustbeautyReviews::WIDGET_NB_COLUMNS);
    updateConfigurationName('TBR_WIDGET_NBREVIEWS', TrustbeautyReviews::WIDGET_NB_COLUMNS);
    updateConfigurationName('TBR_WIDGET_ORDER', TrustbeautyReviews::WIDGET_ORDER);
    updateConfigurationName('TRUSTBEAUTY_APIKEY', TrustbeautyReviews::APIKEY);
    updateConfigurationName('TRUSTBEAUTY_APICHECKED', TrustbeautyReviews::API_CHECKED);
    Configuration::updateGlobalValue(
        TrustbeautyReviews::ORDERS_STATUS,
        json_encode([Configuration::get('PS_OS_DELIVERED')])
    );

    return true;
}

function updateConfigurationName($oldName, $newName)
{
    Db::getInstance()->update(
        'configuration',
        ['name' => pSQL($newName)],
        'name = \'' . pSQL($oldName) . '\''
    );
}

function dropOldTables()
{
    $sql = 'DROP TABLE IF EXISTS `ps_tbr_product`;'; // In old table we has hardcoded ps_ prefix

    Db::getInstance()->execute($sql);
}

function copyDataOldTable()
{
    $newTable = _DB_PREFIX_ . ProductStat::$definition['table'];
    $sqlCopy = "
    INSERT INTO `{$newTable}` (`id_product`, `iso_lang`, `id_shop`, `nb_reviews`, 
    `avg_rating`, `date_add`, `date_upd`) (
        SELECT
            tbr.`id_product`,
            tbr.`iso_lang`,
            tbr.`id_shop`,
            tbr.`nb_reviews`,
            tbr.`avg_rating`,
            tbr.`date_add`,
            tbr.`date_update`
        FROM `ps_tbr_product` tbr
        WHERE tbr.`id_product` > 0
    );
    ALTER TABLE `{$newTable}` ADD UNIQUE KEY `ProductShopLang` (`id_product`,`id_shop`,`iso_lang`);";

    Db::getInstance()->execute($sqlCopy);
}
